#include <ros/ros.h>
#include <geometry_msgs/PoseArray.h>
#include <geometry_msgs/Pose.h>

#include <iostream>
#include <math.h>


using namespace std;
/**
 * This class get the position of detected object and when it receive a move command, it will move the detected object
 * to the target position gotten. It can also change the type of object detected by the detector.
 */
class Arm_controler
{
	geometry_msgs::Pose objectPosition;/**<position of the object*/
	
	ros::NodeHandle nh;/**<ROS node*/
	ros::Publisher pub;/**<ROS publisher for controling the arm*/
	ros::Subscriber subObjectPosition;/**<ROS subscriber for getting the position of the objects*/
	ros::Subscriber subCoordinates;/**<ROS subscriber for getting the coordinates of the target position*/
		
	/**************************************
	 * Zero conversion from kinect to arm 
	 **************************************/
	int X_OFFSET = 0;/**< Distance on the x axis between the origin of the kinect and the origin of the arm in cm*/
	int Y_OFFSET = 0;/**< Distance on the y axis between the origin of the kinect and the origin of the arm in cm*/
	int Z_OFFSET = 0;/**< Distance on the z axis between the origin of the kinect and the origin of the arm in cm*/
    
public:
	/**
	* Class Constructor
	* It manage the ROS Subscriber and Publisher
	*/	
    Arm_controler()
    {
		subObjectPosition = nh.subscribe("/object_detection/red_objects_position", 1000, &Arm_controler::getObjectPosition, this);
		subCoordinates = nh.subscribe("/arm_controler/new_coordinates", 1000, &Arm_controler::getCoordinates, this);
		
		pub = nh.advertise<geometry_msgs::PoseArray>("/arm_controler/arm_command", 1000, this);	
    }

    /**
	 * Class Destructor
	 */
	~Arm_controler()
	{
	}
    
    /**
	 * 
	 * @param msg
	 */
    void getCoordinates(const geometry_msgs::Pose msg)
    {
		geometry_msgs::Pose newCoordinate = msg;
		geometry_msgs::PoseArray objectDisplacement;
		
		objectDisplacement.poses.push_back(objectPosition);
		objectDisplacement.poses.push_back(newCoordinate);
		
		pub.publish(objectDisplacement);
	}
    /**
	 * 
	 * @param msg
	 */
	void  getObjectPosition(const geometry_msgs::PoseArray msg)
	{
		objectPosition = msg.poses[0];
		objectPosition.position.x += X_OFFSET;
		objectPosition.position.y += Y_OFFSET;
		objectPosition.position.z += Z_OFFSET;
    }	
};


int main(int argc, char** argv)
{
	ros::init(argc, argv, "arm_controler");

	Arm_controler ac;

	ros::spin();
}

